#!/bin/bash

### vCard information
FIRSTNAME=Joe
LASTNAME=Blow
TELEPHONE="311 555 1212"
EMAIL=joe@aol.com
PHOTO=./photo.jpg
NOTE=

### Parameters for the final WEBP image to put in the vCard
WEBP_SIZE=60x60
WEBP_SATURATION=59
WEBP_QUALITY=65

### Path to the various utilities we need
CWEBP=cwebp
NDEFTOOL=ndeftool
TAGTOOL="python3 nfcpy/examples/tagtool.py"
 


# Resize / desaturate / compress the original photo
echo "Resizing the photo"
convert $PHOTO -resize $WEBP_SIZE -modulate 100,$WEBP_SATURATION,100 \
	./photo_resized.png

echo "Compressing the photo"
$CWEBP -q $WEBP_QUALITY ./photo_resized.png -o \
	./photo_compressed.webp

# Create the .vcf file
echo "Creating the vCard file"

echo -en "BEGIN:VCARD\r\n" > vcard.vcf
echo -en "VERSION:2.1\r\n" >> vcard.vcf
echo -en "N:$LASTNAME;$FIRSTNAME\r\n" >> vcard.vcf
echo -en "TEL:$TELEPHONE\r\n" >> vcard.vcf
echo -en "EMAIL:$EMAIL\r\n" >> vcard.vcf

B64ENCPHOTO=$(base64 -w0 ./photo_compressed.webp)
B64ENCPHOTO="PHOTO;ENCODING=BASE64;TYPE=WEBP:$B64ENCPHOTO"
echo $B64ENCPHOTO | head -c 78 >> vcard.vcf
for L in $(echo $B64ENCPHOTO | tail -c +79 | sed -e 's/.\{77\}/&\n/g'); do
  echo -en "\r\n $L" >> vcard.vcf
done
echo -en "\r\n" >> vcard.vcf

if [ "$NOTE" ];then
  echo -en "NOTE:$NOTE\r\n" >> vcard.vcf
fi

echo -en "END:VCARD\r\n" >> vcard.vcf

# Dump the size of the .vcf file, for information
VCARDSIZE=$(wc -c < vcard.vcf)
echo "vCard size: $VCARDSIZE bytes"

# Create the NDEF record
echo "Encapsulating the vCard into an NDEF record"
$NDEFTOOL load --pack vcard.vcf tn text/vcard id '' save --force vcard.ndef

# Dump the size of the NDEF file, for information
NDEFSIZE=$(wc -c < vcard.ndef)
echo "NDEF size: $NDEFSIZE bytes"

# Read the tag
echo "Reading the tag"
CARD_INFO=$($TAGTOOL)

# Make sure the tag is writable
if [ ! "$(echo $CARD_INFO | grep -i 'NDEF Capabilities.*writeable *= *yes')" ]; then
  echo "Tag is not writeable!"
  exit -1
fi

# Make sure there's enough room on the tag for the NDEF record
TCAPA=$(echo $CARD_INFO | sed -r 's/^.*NDEF Capabilities.*capacity *= *([0-9]+) byte.*$/\1/')
if [ $TCAPA -lt $NDEFSIZE ]; then
  echo "NDEF size exceeds the tag's capacity ($TCAPA bytes)!"
  exit -1
fi

# Ask the user if they really want to write the vCard on the tag
read -p "Write NDEF onto the tag [Y/N]? " YN
if [ "$YN" != "y" ] && [ "$YN" != "Y" ]; then
  echo "Aborting"
  exit 0
fi

# Write the NDEF file onto the tag
echo "Writing NDEF onto the tag"
$TAGTOOL load vcard.ndef

echo "All done!"
